#    Copyright 2020, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


MOCK_POOL_ID = 'mock-pool-1'
MOCK_MEMBER_ID = 'mock-member-1'
MOCK_LOAD_BALANCER_ID = 'mock-lb-1'
MOCK_COOKIE_NAME = 'mock-cookie-1'
MOCK_LISTENER_ID = 'mock-listener-1'
MOCK_CERT_FILENAME = 'mock-cert-filename-1'
MOCK_CERT_CONTENT = 'mock-cert-content-1'
MOCK_KEY_FILENAME = 'mock-key-filename-1'
MOCK_KEY_CONTENT = 'mock-key-content-1'
MOCK_KEY_PASS = 'mock-key-pass-1'
MOCK_TEMPLATE_NAME = 'mock-template-name-1'
MOCK_CERT_ACTION = 'import'
MOCK_CERT_TYPE = 'pem'
